@@ -4,6 +4,7 @@
 #include <stdint.h>
 #include <stdbool.h>
 #include "jpg_0XC3.h"
+#include "print.h"
 
 unsigned char  readByte(unsigned char *lRawRA, long *lRawPos, long lRawSz) {
     unsigned char ret = 0x00;
@@ -103,27 +104,32 @@ int decodePixelDifference(unsigned char *lRawRA, long *lRawPos, int *lCurrentBit
 
 unsigned char *  decode_JPEG_SOF_0XC3 (const char *fn, int skipBytes, bool verbose, int *dimX, int *dimY, int *bits, int *frames, int diskBytes) {
     //decompress JPEG image named "fn" where image data is located skipBytes into file. diskBytes is compressed size of image (set to 0 if unknown)
-    #define abortGoto() free(lRawRA); return NULL;
+    //next line breaks MSVC
+    // #define abortGoto(...) ({printError(__VA_ARGS__); free(lRawRA); return NULL;})
+    #define abortGoto(...) do {printError(__VA_ARGS__); free(lRawRA); return NULL;} while(0)
     unsigned char *lImgRA8 = NULL;
     FILE *reader = fopen(fn, "rb");
-    fseek(reader, 0, SEEK_END);
+    int lSuccess = fseek(reader, 0, SEEK_END);
     long lRawSz = ftell(reader)- skipBytes;
     if ((diskBytes > 0) && (diskBytes < lRawSz)) //only if diskBytes is known and does not exceed length of file
         lRawSz = diskBytes;
-    if (lRawSz <= 8) {
-        printf("Error opening %s\n", fn);
+    if ((lSuccess != 0) || (lRawSz <= 8)) {
+        printError("Unable to load 0XC3 JPEG %s\n", fn);
+        return NULL; //read failure
+    }
+    lSuccess = fseek(reader, skipBytes, SEEK_SET); //If successful, the function returns zero
+    if (lSuccess != 0) {
+        printError("Unable to open 0XC3 JPEG  %s\n", fn);
         return NULL; //read failure
     }
-    fseek(reader, skipBytes, SEEK_SET);
     unsigned char *lRawRA = (unsigned char*) malloc(lRawSz);
-    fread(lRawRA, 1, lRawSz, reader);
+    size_t lSz = fread(lRawRA, 1, lRawSz, reader);
     fclose(reader);
-    if ((lRawRA[0] != 0xFF) || (lRawRA[1] != 0xD8) || (lRawRA[2] != 0xFF)) {
-        printf("Error: JPEG signature 0xFFD8FF not found at offset %d of %s\n", skipBytes, fn);
-        abortGoto();//goto abortGoto; //signature failure http://en.wikipedia.org/wiki/List_of_file_signatures
+    if ((lSz < lRawSz) || (lRawRA[0] != 0xFF) || (lRawRA[1] != 0xD8) || (lRawRA[2] != 0xFF)) {
+        abortGoto("JPEG signature 0xFFD8FF not found at offset %d of %s\n", skipBytes, fn);//signature failure http://en.wikipedia.org/wiki/List_of_file_signatures
     }
     if (verbose)
-        printf("JPEG signature 0xFFD8FF found at offset %d of %s\n", skipBytes, fn);
+        printMessage("JPEG signature 0xFFD8FF found at offset %d of %s\n", skipBytes, fn);
     //next: read header
     long lRawPos = 2; //Skip initial 0xFFD8, begin with third byte
     //long lRawPos = 0; //Skip initial 0xFFD8, begin with third byte
@@ -138,8 +144,7 @@ unsigned char *  decode_JPEG_SOF_0XC3 (const char *fn, int skipBytes, bool verbo
         do {
             btS1 = readByte(lRawRA, &lRawPos, lRawSz);
             if (btS1 != 0xFF) {
-                printf("JPEG header tag must begin with 0xFF\n");
-                abortGoto(); //goto abortGoto;
+                abortGoto("JPEG header tag must begin with 0xFF\n");
             }
             btMarkerType =  readByte(lRawRA, &lRawPos, lRawSz);
             if ((btMarkerType == 0x01) || (btMarkerType == 0xFF) || ((btMarkerType >= 0xD0) && (btMarkerType <= 0xD7) ) )
@@ -148,10 +153,11 @@ unsigned char *  decode_JPEG_SOF_0XC3 (const char *fn, int skipBytes, bool verbo
         } while ((lRawPos < lRawSz) && (btMarkerType == 0));
         uint16_t lSegmentLength = readWord (lRawRA, &lRawPos, lRawSz); //read marker length
         long lSegmentEnd = lRawPos+(lSegmentLength - 2);
-        if (lSegmentEnd > lRawSz)
-            abortGoto(); //goto abortGoto;
+        if (lSegmentEnd > lRawSz) {
+            abortGoto("Segment larger than image\n");
+        }
         if (verbose)
-            printf("btMarkerType %#02X length %d@%ld\n", btMarkerType, lSegmentLength, lRawPos);
+            printMessage("btMarkerType %#02X length %d@%ld\n", btMarkerType, lSegmentLength, lRawPos);
         if ( ((btMarkerType >= 0xC0) && (btMarkerType <= 0xC3)) || ((btMarkerType >= 0xC5) && (btMarkerType <= 0xCB)) || ((btMarkerType >= 0xCD) && (btMarkerType <= 0xCF)) )  {
             //if Start-Of-Frame (SOF) marker
             SOFprecision = readByte(lRawRA, &lRawPos, lRawSz);
@@ -160,18 +166,16 @@ unsigned char *  decode_JPEG_SOF_0XC3 (const char *fn, int skipBytes, bool verbo
             SOFnf = readByte(lRawRA, &lRawPos, lRawSz);
             //SOFarrayPos = lRawPos;
             lRawPos = (lSegmentEnd);
-            if (verbose) printf(" [Precision %d X*Y %d*%d Frames %d]\n", SOFprecision, SOFxdim, SOFydim, SOFnf);
+            if (verbose) printMessage(" [Precision %d X*Y %d*%d Frames %d]\n", SOFprecision, SOFxdim, SOFydim, SOFnf);
             if (btMarkerType != 0xC3) { //lImgTypeC3 = true;
-                printf("This JPEG decoder can only decompress lossless JPEG ITU-T81 images (SoF must be 0XC3, not %#02X)\n",btMarkerType );
-                abortGoto(); //goto abortGoto;
+                abortGoto("This JPEG decoder can only decompress lossless JPEG ITU-T81 images (SoF must be 0XC3, not %#02X)\n",btMarkerType );
             }
             if ( (SOFprecision < 1) || (SOFprecision > 16) || (SOFnf < 1) || (SOFnf == 2) || (SOFnf > 3)
                 || ((SOFnf == 3) &&  (SOFprecision > 8))   ) {
-                printf("Scalar data must be 1..16 bit, RGB data must be 8-bit (%d-bit, %d frames)\n", SOFprecision, SOFnf);
-                abortGoto(); //goto abortGoto;
+                abortGoto("Scalar data must be 1..16 bit, RGB data must be 8-bit (%d-bit, %d frames)\n", SOFprecision, SOFnf);
             }
         } else if (btMarkerType == 0xC4) {//if SOF marker else if define-Huffman-tables marker (DHT)
-            if (verbose) printf(" [Huffman Length %d]\n", lSegmentLength);
+            if (verbose) printMessage(" [Huffman Length %d]\n", lSegmentLength);
             int lFrameCount = 1;
             do {
                 uint8_t DHTnLi = readByte(lRawRA, &lRawPos, lRawSz ); //we read but ignore DHTtcth.
@@ -181,10 +185,11 @@ unsigned char *  decode_JPEG_SOF_0XC3 (const char *fn, int skipBytes, bool verbo
                     l[lFrameCount].DHTliRA[lInc] = readByte(lRawRA, &lRawPos, lRawSz);
                     DHTnLi = DHTnLi +  l[lFrameCount].DHTliRA[lInc];
                     if (l[lFrameCount].DHTliRA[lInc] != 0) l[lFrameCount].MaxHufSi = lInc;
+                    if (verbose) printMessage("DHT has %d combinations with %d bits\n", l[lFrameCount].DHTliRA[lInc], lInc);
+                    
                 }
                 if (DHTnLi > 17) {
-                    printf("Huffman table corrupted.\n");
-                    abortGoto(); //goto abortGoto;
+                    abortGoto("Huffman table corrupted.\n");
                 }
                 int lIncY = 0; //frequency
                 for (int lInc = 0; lInc <= 31; lInc++) {//lInc := 0 to 31 do begin
@@ -200,11 +205,11 @@ unsigned char *  decode_JPEG_SOF_0XC3 (const char *fn, int skipBytes, bool verbo
                             btS1 = readByte(lRawRA, &lRawPos, lRawSz);
                             l[lFrameCount].HufVal[lIncY] = btS1;
                             l[lFrameCount].MaxHufVal = btS1;
-                            if ((btS1 >= 0) && (btS1 <= 16))
+                            if (verbose) printMessage("DHT combination %d has a value of %d\n", lIncY, btS1);
+                            if (btS1 <= 16) //unsigned ints ALWAYS >0, so no need for(btS1 >= 0)
                                 l[lFrameCount].HufSz[lIncY] = lInc;
                             else {
-                                printf("Huffman size array corrupted.\n");
-                                abortGoto(); //goto abortGoto;
+                                abortGoto("Huffman size array corrupted.\n");
                             }
                         }
                     }
@@ -228,15 +233,14 @@ unsigned char *  decode_JPEG_SOF_0XC3 (const char *fn, int skipBytes, bool verbo
                 } while (K <= DHTnLi);
                 //if (verbose)
                 //    for (int j = 1; j <= DHTnLi; j++)
-                //        printf(" [%d Sz %d Code %d Value %d]\n", j, l[lFrameCount].HufSz[j], l[lFrameCount].HufCode[j], l[lFrameCount].HufVal[j]);
+                //        printMessage(" [%d Sz %d Code %d Value %d]\n", j, l[lFrameCount].HufSz[j], l[lFrameCount].HufCode[j], l[lFrameCount].HufVal[j]);
                 lFrameCount++;
             } while ((lSegmentEnd-lRawPos) >= 18);
             lnHufTables = lFrameCount - 1;
             lRawPos = (lSegmentEnd);
-            if (verbose) printf(" [FrameCount %d]\n", lnHufTables);
+            if (verbose) printMessage(" [FrameCount %d]\n", lnHufTables);
         } else if (btMarkerType == 0xDD) {  //if DHT marker else if Define restart interval (DRI) marker
-            printf("This image uses Restart Segments - please contact Chris Rorden to add support for this format.\n");
-            abortGoto(); //goto abortGoto;
+            abortGoto("btMarkerType == 0xDD: unsupported Restart Segments\n");
             //lRestartSegmentSz = ReadWord(lRawRA, &lRawPos, lRawSz);
             //lRawPos = lSegmentEnd;
         } else if (btMarkerType == 0xDA) {  //if DRI marker else if read Start of Scan (SOS) marker
@@ -257,17 +261,17 @@ unsigned char *  decode_JPEG_SOF_0XC3 (const char *fn, int skipBytes, bool verbo
             SOSahal = readByte(lRawRA, &lRawPos, lRawSz); //lower 4bits= pointtransform
             SOSpttrans = SOSahal & 16;
             if (verbose)
-                printf(" [Predictor: %d Transform %d]\n", SOSss, SOSahal);
+                printMessage(" [Predictor: %d Transform %d]\n", SOSss, SOSahal);
             lRawPos = (lSegmentEnd);
         } else  //if SOS marker else skip marker
             lRawPos = (lSegmentEnd);
     } while ((lRawPos < lRawSz) && (btMarkerType != 0xDA)); //0xDA=Start of scan: loop for reading header
     //NEXT: Huffman decoding
     if (lnHufTables < 1) {
-        printf("Decoding error: no Huffman tables.\n");
-        abortGoto(); //goto abortGoto;
+        abortGoto("Decoding error: no Huffman tables.\n");
     }
     //NEXT: unpad data - delete byte that follows $FF
+    int lIsRestartSegments = 0;
     long lIncI = lRawPos; //input position
     long lIncO = lRawPos; //output position
     do {
@@ -277,10 +281,14 @@ unsigned char *  decode_JPEG_SOF_0XC3 (const char *fn, int skipBytes, bool verbo
                 lIncI = lIncI+1;
             else if (lRawRA[lIncI+1] == 0xD9)
                 lIncO = -666; //end of padding
+            else
+                lIsRestartSegments = lRawRA[lIncI+1];
         }
         lIncI++;
         lIncO++;
     } while (lIncO > 0);
+    if (lIsRestartSegments != 0) //detects both restart and corruption https://groups.google.com/forum/#!topic/comp.protocols.dicom/JUuz0B_aE5o
+        printWarning("Detected restart segments, decompress with dcmdjpeg or gdcmconv 0xFF%02X.\n", lIsRestartSegments);
     //NEXT: some RGB images use only a single Huffman table for all 3 colour planes. In this case, replicate the correct values
     //NEXT: prepare lookup table
     for (int lFrameCount = 1; lFrameCount <= lnHufTables; lFrameCount ++) {
@@ -306,7 +314,7 @@ unsigned char *  decode_JPEG_SOF_0XC3 (const char *fn, int skipBytes, bool verbo
                         }
                     } else
                         l[lFrameCount].LookUpRA[k] = lHufVal; //SSSS
-                    //printf("Frame %d SSSS %d Size %d Code %d SHL %d EmptyBits %ld\n", lFrameCount, lHufRA[lFrameCount][lIncY].HufVal, lHufRA[lFrameCount][lIncY].HufSz,lHufRA[lFrameCount][lIncY].HufCode, k, lInc);
+                    //printMessage("Frame %d SSSS %d Size %d Code %d SHL %d EmptyBits %ld\n", lFrameCount, lHufRA[lFrameCount][lIncY].HufVal, lHufRA[lFrameCount][lIncY].HufSz,lHufRA[lFrameCount][lIncY].HufCode, k, lInc);
                 } //Set SSSS
             } //Length of size lInc > 0
         } //for lInc := 1 to 8
@@ -489,6 +497,6 @@ unsigned char *  decode_JPEG_SOF_0XC3 (const char *fn, int skipBytes, bool verbo
     *dimY = SOFydim;
     *frames = SOFnf;
     if (verbose)
-        printf("JPEG ends %ld@%ld\n", lRawPos, lRawPos+skipBytes);
+        printMessage("JPEG ends %ld@%ld\n", lRawPos, lRawPos+skipBytes);
     return lImgRA8;
 }// decode_JPEG_SOF_0XC3()
\ No newline at end of file
